%
% Decentralized Reconstruction from Compressive Random Projections
% Driven by Principal Components
% Copyright (C) 2014-2015  James E. Fowler
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
%

function [X_hat, Psi] = Decentralized_Decoder(Y_tilde, P, L, Graph_WJ)

M = length(P);
[N K] = size(P{1});

X_hat = zeros(N, M);

for m = 1:M
  Psi{m} = zeros(N, L);
end

Y_tilde2 = Y_tilde;

for l = 1:L

  for m = 1:M
    X_hat(:, m) = P{m} * Y_tilde2(:, m);
  end
  
  Sigma_hat = CovarianceConsensus(Graph_WJ, X_hat);
  
  for m = 1:M
    [Psi{m}(:, l) lambda] = eigs(Sigma_hat{m}, 1);
    X_hat(:, m) = Psi{m} * pinv(P{m}' * Psi{m}) * Y_tilde(:, m); 
    Y_tilde2(:, m) = Y_tilde(:, m) - P{m}' * X_hat(:, m);
  end

end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function Sigma = CovarianceConsensus(Graph_WJ, X)

[N M] = size(X);

I = triu(ones(N, N));

Z = zeros(M, N*(N+1)/2);

index = 1;
for col = 1:N
  for row = 1:col
    Z(:, index) = X(row, :) .* X(col, :);
    index = index + 1;
  end
end

% disp('    Average consensus...');
% tic
Z = AverageConsensus(Graph_WJ, Z);
% toc

for m = 1:M
  Sigma1 = zeros(N, N);
  Sigma1(I == 1) = Z(m, :);
  Sigma1_t = Sigma1';
  Sigma1(I == 0) = Sigma1_t(I == 0);
  Sigma{m} = Sigma1;
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function Z_mean = AverageConsensus(WJ, Z)

Z_mean = WJ * Z;







