%
% Decentralized Reconstruction from Compressive Random Projections
% Driven by Principal Components
% Copyright (C) 2014-2015  James E. Fowler
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
%

function L = GenerateRandomGraph(num_nodes, expected_node_degree)

L = sparse(num_nodes, num_nodes);

% Linearly connected graph - ensure one connected component
for row = 1:(num_nodes - 1)
  L(row, row + 1) = -1;
  L(row + 1, row) = -1;
end
L(num_nodes, 1) = -1;
L(1, num_nodes) = -1;

% Select random edges
max_num_edges = num_nodes * (num_nodes - 1) / 2;

edges = randperm(max_num_edges);
edges = edges(1:ceil(max_num_edges * (expected_node_degree - 2) / num_nodes));

edge_flag = zeros(1, max_num_edges);
edge_flag(edges) = 1;

index = 1;
for row = 1:num_nodes
  for col = (row + 1):num_nodes
    if (edge_flag(index) == 1)
      L(row, col) = -1;
      L(col, row) = -1;
    end
    index = index + 1;
  end
end

L = spdiags((-sum(L))', 0, L);
