%
% Decentralized Reconstruction from Compressive Random Projections
% Driven by Principal Components
% Copyright (C) 2014-2015  James E. Fowler
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
%

function Graph_W = GenerateWeightMatrix(Graph_L, weight_type)

num_nodes = size(Graph_L, 2);

Graph_W = sparse(num_nodes, num_nodes);

Graph_L_diag = full(diag(Graph_L));

switch weight_type
  case 'maxdegree'
    max_degree = max(Graph_L_diag);
    alpha = 1/max_degree;
    Graph_W = speye(num_nodes) - alpha*Graph_L;

  case 'bestconstant'
    lambda = sort(eig(Graph_L), 'descend');
    alpha = 2/(lambda(1) + lambda(num_nodes - 1));
    Graph_W = speye(num_nodes) - alpha*Graph_L;
    
  case 'localdegree'
    for row = 1:num_nodes
      I = (Graph_L(row, :) == -1);
      Graph_W(row, I) = 1 ./ max(full(Graph_L_diag(row)), Graph_L_diag(I)');

      Graph_W(row, row) = 1 - sum(Graph_W(row, :));
    end
  otherwise
    disp('Invalid weight_type');
    Graph_W = [];
end
