%
% Decentralized Reconstruction from Compressive Random Projections
% Driven by Principal Components
% Copyright (C) 2014-2015  James E. Fowler
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
%

function S = SNR(x, x_hat)

if (isvector(x) && isvector(x_hat))
  if (length(x) ~= length(x_hat))
    error('Vector x and x_hat must both have the same length');
  end
  x = x(:);
  x_hat = x_hat(:);
else
  if (~isvector(x) && ~isvector(x_hat))
    if (size(x) ~= size(x_hat))
      error('Sizes of matrices x and x_hat must be the same');
    end
  else
    error('x and x_hat must be either both vectors or both matrices');
  end
end

diff = x - x_hat;
mse = mean(diff .^ 2);

variance = var(x);

S = 10*log10(variance ./ mse);
