%
% Decentralized Reconstruction from Compressive Random Projections
% Driven by Principal Components
% Copyright (C) 2014-2015  James E. Fowler
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
%

function run_experiment()

load random_graph.10000.3.mat;

L = 30;

%
% Choose the AVIRIS dataset
%
% filename = 'cuprite';
filename = 'moffett';
% filename = 'jasper';

fid = fopen([filename '.dat'], 'r', 'ieee-be');
X = fread(fid, [224 M], 'float32');
fclose(fid);
[N M] = size(X);
Sigma = X * X' / M;
[W SS VV] = svd(Sigma);

projection_matrix_file = ['projections.' num2str(M) '.' num2str(N) '.mat'];

subrate = 0.3;
K = round(subrate * N);
Js = [1 10 20 50 75 100];
% Js = [1 100];

disp(['Running ' filename '...']);
disp('Running centralized encoder...');
P = GenerateProjections(N, K, M, projection_matrix_file);
Y_tilde = Encoder(X, P);

disp('Running centralized decoder...');
[X_hat, Psi_centralized] = Centralized_Decoder(Y_tilde, P, L);

D_centralized = mean(SNR(X, X_hat));
disp(['  Centralized SNR = ' num2str(D_centralized) ' dB']);

for l = 1:L
  S_centralized(l) = SAM(Psi_centralized(:, l), W(:, l));
end

for index1 = 1:length(Js)

  disp(['Num iterations = ' num2str(Js(index1))]);

  J = Js(index1);
  
  Graph_WJ = GraphPower(Graph_W, J);

  disp('   Running decentralized decoder...');
  [X_hat_decentralized, Psi_decentralized] = ...
      Decentralized_Decoder(Y_tilde, P, L, Graph_WJ);
  
  D_decentralized(index1) = mean(SNR(X, X_hat_decentralized));
  disp(['  Decentralized SNR = ' num2str(D_decentralized(index1)) ' dB']);
  
  s = zeros(M, L);
  for m = 1:M
    s(m, :) = SAM(Psi_decentralized{m}(:, 1:L), W(:, 1:L));
  end
  S_decentralized(index1, 1:L) = mean(s);
  
end

save([filename '.' num2str(M) '.' num2str(subrate * 100) '.results.mat'], ...
    'D_centralized', 'S_centralized', ...
    'D_decentralized', 'S_decentralized', ...
    'Js');

for l = 1:L
  disp(['  Centralized average SAM eigenvector ' num2str(l) ' = ' ...
        num2str(S_centralized(l))]);
end  

for index1 = 1:length(Js)
  disp(['Num iterations = ' num2str(Js(index1))]);
  
  for l = 1:L
    disp(['  Decentralized average SAM eigenvector ' num2str(l) ' = ' ...
          num2str(S_decentralized(index1, l))]);
  end
end
