%
% Decentralized Reconstruction from Compressive Random Projections
% Driven by Principal Components
% Copyright (C) 2014-2015  James E. Fowler
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
%

function run_make_plot()

% format = 'pdf';
format = 'eps';

close all;

filenames = {'cuprite', 'moffett', 'jasper'};

M = 10000;
subrate = 0.3;
L = 3;

figure_num = 1;

for index = 1:length(filenames)

  filename = filenames{index};

  load([filename '.' num2str(M) '.' num2str(subrate * 100) ...
	'.results.mat']);
  
  figure(figure_num);
  figure_num = figure_num + 1;
  
  plot(Js, D_decentralized, 'k-x', 'LineWidth', 2);
  
  hold on;
  grid on;
  
  plot([Js(1) Js(end)], ...
      [D_centralized D_centralized], 'k--', 'LineWidth', 2);
  
  ylabel('Average SNR (dB)');
  xlabel('Number of consensus iterations, J');
  if (strcmp(format, 'pdf'))
    title([filename ', M = ' num2str(M) ', subrate = ' ...
	  num2str(subrate)]);
  end

  legend('Decentralized', 'Centralized', 'Location', 'SouthEast');
  
  print(['-d' format], [filename '.snr.' format]);

  for l = 1:L

    figure(figure_num);
    figure_num = figure_num + 1;
    
    plot(Js, S_decentralized(:, l), 'k-x', 'LineWidth', 2);
    
    hold on;
    grid on;
    
    plot([Js(1) Js(end)], ...
	[S_centralized(l) S_centralized(l)], 'k--', 'LineWidth', 2);
    
    ylabel(['\xi_' num2str(l) ' (degrees)']);
    xlabel('Number of consensus iterations, J');
    if (strcmp(format, 'pdf'))
      title([filename ', M = ' num2str(M) ', subrate = ' ...
	    num2str(subrate) ', eigenvector = ' num2str(l)]);
    end
    
    legend('Decentralized', 'Centralized', 'Location', 'NorthEast');

    print(['-d' format], [filename '.sam.' num2str(l) '.' format]);
    
  end
  
end
